package common

// EDI 文件类型和对象
const (
	EDI_TOD     = "TOD"
	EDI_SEQ     = "SEQ"
	EDI_REORDER = "REORDER"
	EDI_CALLOFF = "CALLOFF"

	EDI_PARSE_YES   = 1
	EDI_PARSE_NO    = 0
	EDI_PARSE_ERROR = 9

	// TOD文件数据行对象
	TOD_OBJ = "UNB,UNH,BGM,DTM,NAD,SEQ,GIR,LIN,PIA,QTY,UNT,UNZ"
	// SEQ文件数据行对象
	SEQ_OBJ = "UNB,UNH,BGM,DTM,FTX,NAD,SEQ,GIR,LOC,UNT,UNZ"
	// Reorder文件数据行对象
	REORDER_OBJ = "UNB,UNH,BGM,DTM,REF,NAD,FTX,SEQ,GIR,LIN,PIA,QTY,UNT,UNZ"
	// Calloff文件数据行对象
	CALLOFF_OBJ = "UNB,UNH,BGM,DTM,REF,NAD,LOC,SEQ,GIR,UNT,UNZ"
)

const (
	OBJ_ACTIVE   = 1 // 激活
	OBJ_DEACTIVE = 0 // 未激活
)

// 属性类型定义常量
const (
	ATCOD_TYPE_PART = 1 // 零件属性
	ATCOD_TYPE_PROD = 2 // 生产属性
)

// 供应组类型定义
const (
	SG_TYPE_NORMAL  = "F" // 客户供应组
	SG_TYPE_CHILD   = "C" // 子供应组
	SG_TYPE_NEUTRAL = "N" // 共用件供应组
)

// 客户订单状态常量
const (
	CO_STATUS_UNPLANED = 10 // 未计划
	CO_STATUS_PLANNED  = 20 // 已计划
	CO_STATUS_RELEASED = 26 // 已下达
	CO_STATUS_RUNNING  = 40 // 生产中
	CO_STATUS_CLOSED   = 80 // 已完成
	CO_STATUS_LOCKED   = 95 // 锁定
	CO_STATUS_CANCELED = 98 // 取消
	// 客户订单解析状态，如果验证条件满足，则为OK，不满足，则报错为NG
	CO_PARSE_STATUS_OK      = "OK"
	CO_PARSE_STATUS_INIT    = "IT"
	CO_PARSE_STATUS_ERROR   = "ER"
	CO_PARSE_STATUS_RECYCLE = "RC"
	// 检查错误类型
	CO_CHECK_NOTEXIST           = 101 // 客户订单不存在
	CO_CEHCK_STATUS_NOT_SATISFY = 102 // 客户订单状态不满足
	// 验证错误类型
	CO_VERIFY_PARTFAMILY_NOTEXIST  = 201 // 零件族不存在
	CO_VERIFY_SUPPLYGROUP_NOTEXIST = 202 // 供应组不存在
	CO_VERIFY_PART_NOTEXIST        = 203 // 零件不存在
	CO_VERIFY_ATCOD_NOTEXIST       = 204 // 属性不存在
	CO_VERIFY_ATCOD_VALUE_CONFLICT = 205 // 属性值冲突
	CO_VERIFY_PF_ATCOD_NOT_SATISFY = 206 // 零件族的强制属性没有满足
	CO_VERIFY_PARTRULE_NOTEXIST    = 207 // 零件分配规则不存在
	CO_VERIFY_SG_ATCOD_NOT_SATISFY = 208 // 供应组的强制属性没有满足
	//重复订单
	REPEATORDER = 301 // TOD SEQ+3的重复订单
)

//calloff Errortype
const (
	CALLOFF_CHECK_SEQUENCE  = 901
	CALLOFF_ORDER_NOTEXIST  = 902
	CALLOFF_ORDER_NOTSTATUS = 903
	CALLOFF_ORDER_NOTSPEC   = 904
)

// 生产订单状态常量
const (
	WO_STATUS_UNPLANED = 10 // 未计划
	WO_STATUS_PLANNED  = 20 // 已计划
	WO_STATUS_RELEASED = 26 // 已下达
	WO_STATUS_RUNNING  = 40 // 生产中
	WO_STATUS_CLOSED   = 80 // 已完成
	WO_STATUS_LOCKED   = 95 // 锁定
	WO_STATUS_CANCELED = 98 // 取消
	WO_STATUS_PACKED   = 1  // 已包装
	WO_STATUS_UNPACKED = 0  // 未包装
	WO_STATUS_UNRUNMRP = 0
	WO_STATUS_RUNMRP   = 1

	WO_TYPE_SEQ      = "SEQ"
	WO_TYPE_REORDER  = "REO"
	WO_TYPE_INTERNAL = "INT"

	BATORD_TYPE_SEQ    = "SEQ"
	BATORD_TYPE_MANUAL = "MANUAL"
)

// 拣料状态
const (
	PK_STATUS_UNPICKED  = 0  // 未拣料
	PK_STATUS_PICKED    = 1  // 已拣料
	PK_STATUS_HANDLED   = 2  // 已处理（不需要拣料）
	PKO_STATUS_PLANNED  = 20 // 已计划
	PKO_STATUS_RELEASED = 26 // 已下达
	PKO_STATUS_RUNNING  = 40 // 生产中
	PKO_STATUS_CLOSED   = 80 // 已完成
	PKO_STATUS_UNLOADED = 0  // 未装车
	PKO_STATUS_LOADED   = 1  // 已装车
)

//发运车状态
const (
	SPO_STATUS_UNPLANNED = 10 // 未计划
	SPO_STATUS_PLANNED   = 20 // 已计划（指定模板）
	SPO_STATUS_RELEASED  = 26 // 已下达（完成填充）
	SPO_STATUS_RUNNING   = 40 // 装载中
	SPO_STATUS_CLOSED    = 80 // 已完成
)

// 包装排序方式
const (
	PK_SORT_FORWARD   = "FORWARD"   // 按位置号正序排列
	PK_SORT_BACKWARD  = "BACKWARD"  // 按位置号倒序排列
	PK_SORT_MANUALSEQ = "MANUALSEQ" // 按人工指定位置排序
)

// 验证规则
const (
	VERIFY_RULE_ATVAL_UNIQ = "V_ATV_UNIQ" // 校验属性值的唯一性
	VERIFY_RULE_ATVAL_DISP = "V_ATV_DISP" // 显示属性值
)

// 产线相关设置
const (
	// 产线类型
	LINE_TYPE_MANUAL = "M"
	LINE_TYPE_AUTO   = "A"
	LINE_TYPE_VENDOR = "V"
	// 产线计划模式
	LINE_PLAN_IN_SEQ   = "SEQ"
	LINE_PLAN_IN_BATCH = "BATCH"
	// 任务队列排序方式
	LINE_QUEUE_SORT_IN_SEQ  = "SEQ"
	LINE_QUEUE_SORT_IN_SWET = "SWET"
	// 混线排序方式
	LINE_MIXSORT_IN_SWET       = "SWET"
	LINE_MIXSORT_IN_SWET_RATIO = "SWET_RATIO"
	// 任务派工方式
	LINE_REL_BY_QTY      = "BY_QTY"
	LINE_REL_BY_TIME     = "BY_TIME"
	LINE_REL_BY_DURATION = "BY_DURA"
	LINE_REL_BY_OEMSEQ   = "BY_OEMSEQ"
)

// BOM相关
const (
	BOM_ART_MANUFACTURE = "MAF" // 自制件
	BOM_ART_PURCHASE    = "PCE" // 采购件
)

// EDI文件读取间隔时间，以秒计算
const (
	TOD_READ_INTERVAL     = 5
	SEQ_READ_INTERVAL     = 5
	REORDER_READ_INTERVAL = 5
	CALLOFF_READ_INTERVAL = 5
)

// 打印消息类型和状态
const (
	MSG_TYPE_CARMODEL   = "CARMODEL"
	MSG_TYPE_DEMAND     = "DEMAND"
	MSG_TYPE_PICK       = "PICK"
	MSG_TYPE_PACK       = "PACK"
	MSG_TYPE_SHIP       = "SHIP"
	MSG_TYPE_BBAASN     = "BBAASN"
	MSG_STATUS_PLAN     = 20
	MSG_STATUS_FINISHED = 80
	MSG_STATUS_PENDING  = 95
	MSG_EVENT_PRINT     = "PRINT"
	MSG_EVENT_NEW       = "NEW"
	MSG_EVENT_PARSE     = "PARSE"
	MSG_EVENT_CLOSE     = "CLOSE"
)

// 打印消息变量类型
const (
	PRN_VAR_TYP_VALUE       = "VALUE"
	PRN_VAR_TYP_VARIABLE    = "VARIABLE"
	PRN_VAR_TYP_VARSET      = "VARSET"
	PRN_VAR_TYP_FORMULAR    = "FORMULAR"
	PRN_VAR_TYP_LOOPVAR     = "LOOPVAR"
	PRN_VAR_TYP_SYSVAR      = "SYSVAR"
	PRN_VAR_TYP_2DROW       = "2DROW"
	PRN_VAR_TYP_2DCOL       = "2DCOL"
	PRN_VAR_TYP_2DCELL      = "2DCELL"
	PRN_HEAD_STATUS_NEW     = "N"
	PRN_HEAD_STATUS_CLOSE   = "C"
	PRN_HEAD_STATUS_REPRINT = "R"
)

// 拣料模板
const (
	PKT_COUNT_MODE_BY_WO  = "WORD"
	PKT_COUNT_MODE_BY_QTY = "TQTY"
	PKT_ADAPT_BY_ATTR     = "A"
	PKT_ADAPT_BY_PART     = "P"
)

// 需求相关
const (
	DEMAND_TYPE_FIRM     = "FIRM"
	DEMAND_TYPE_FORECAST = "FORECAST"
	DEMAND_TYPE_SEQ      = "SEQ"
)

// 车型相关
const (
	CM_SPEC_UNAPPROVED = 0
	CM_SPEC_APPROVED   = 1
	CM_PARSE_OK        = "OK"
	CM_PARSE_ERROR     = "ER"
)
