package db

import (
	"errors"
	"github.com/go-xorm/xorm"
	"LAPP_PRN_Service/common"
	"xorm.io/core"
)

/******************************************************************************
 *
 * @Struct Name              : PrintHead
 *-----------------------------------------------------------------------------
 *
 * @Description              : 打印头表
 *
 * @Author                   : Lou wenzhi
 *
 * @Date                     : 2021-04-13 13:08:46
 *
 ******************************************************************************/
type PrintHead struct {
	PlantNr        int             `xorm:"pk int 'PlantNr'" json:"PrintHead-PlantNr"`
	PrintHeadId    string          `xorm:"pk varchar(40) 'PrintHeadId'" json:"PrintHead-PrintHeadId"`
	PrinterId      string          `xorm:"varchar(18) 'PrinterId' not null" json:"PrintHead-PrinterId"`
	PrintFileType  string          `xorm:"varchar(10) 'PrintFileType' not null" json:"PrintHead-PrintFileType"`
	PrintEventType string          `xorm:"varchar(10) 'PrintEventType' not null" json:"PrintHead-PrintEventType"`
	PrintObjId     string          `xorm:"varchar(40) 'PrintObjId' not null" json:"PrintHead-PrintObjId"`
	PrintObjType   string          `xorm:"varchar(12) 'PrintObjType' not null" json:"PrintHead-PrintObjType"`
	Status         string          `xorm:"varchar(1) 'Status' not null" json:"PrintHead-Status"`
	TemplateFile   string          `xorm:"varchar(40) 'TemplateFile' not null" json:"PrintHead-TemplateFile"`
	TermId         string          `xorm:"varchar(30) 'TermId' not null" json:"PrintHead-TermId"`
	WebAddress     string          `xorm:"varchar(40) 'WebAddress' not null" json:"PrintHead-WebAddress"`
	PrintCopies    int             `xorm:"smallint 'PrintCopies' not null" json:"PrintHead-PrintCopies"`
	PrintedTime    string          `xorm:"varchar(14) 'PrintedTime' not null" json:"PrintHead-PrintedTime"`
	PrintedCopies  int             `xorm:"smallint 'PrintedCopies' not null" json:"PrintHead-PrintedCopies"`
	Ival1          int             `xorm:"int 'ival1' not null" json:"PrintHead-ival1"`
	Ival2          int             `xorm:"int 'ival2' not null" json:"PrintHead-ival2"`
	Cval1          string          `xorm:"varchar(60) 'cval1' not null" json:"PrintHead-cval1"`
	Cval2          string          `xorm:"varchar(60) 'cval2' not null" json:"PrintHead-cval2"`
	LastModify     common.DateTime `xorm:"datetime 'LastModify' not null updated" json:"PrintHead-LastModify"`
	LastUser       string          `xorm:"nvarchar(20) 'LastUser' not null" json:"PrintHead-LastUser"`
	CreateTime     common.DateTime `xorm:"datetime 'CreateTime' not null created" json:"PrintHead-CreateTime"`
	Itemlst        []PrintDetail   `xorm:"-"`
}

func (t *PrintHead) TableName() string {
	return "PrintHead"
}

//增
func (t *PrintHead) Add() error {
	e := G_DbEngine
	printheadtab := new(PrintHead)
	affw, err := e.Table("PrintHead").ID(core.PK{PlantNr, t.PrintHeadId}).Count(printheadtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("PrintHead").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//增
func (t *PrintHead) Insert(session *xorm.Session) error {
	printheadtab := new(PrintHead)
	affw, err := session.Table("PrintHead").ID(core.PK{PlantNr, t.PrintHeadId}).Count(printheadtab)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = session.Table("PrintHead").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *PrintHead) Del() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{PlantNr, t.PrintHeadId}).Delete(&PrintHead{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *PrintHead) Update() bool {
	e := G_DbEngine
	_, err := e.ID(core.PK{PlantNr, t.PrintHeadId}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//更新指定字段
func (t *PrintHead) UpdateFields(session *xorm.Session, fields string) (err error) {
	if _, err = session.Table("PrintHead").ID(core.PK{PlantNr, t.PrintHeadId}).Cols(fields).Update(t); err != nil {
		return
	}
	return
}

//查
func (t *PrintHead) SelectOne() (data PrintHead, err error) {
	var (
		itemlst []PrintDetail
	)
	e := G_DbEngine
	_, err = e.Table("PrintHead").ID(core.PK{PlantNr, t.PrintHeadId}).Get(&data)
	if err != nil {
		return
	}
	// 获取明细列表
	if err = e.Table("PrintDetail").Where("PlantNr = ? and PrintHeadId = ?",
		PlantNr, t.PrintHeadId).OrderBy("PrintHeadId").Find(&itemlst); err != nil {
		return
	}
	data.Itemlst = itemlst
	return data, nil
}

// 获取指定类型的开口打印头
func (t *PrintHead) GetOpenSheetOnes() (datalst []PrintHead, err error) {
	var (
		i           int
		itemlst     []PrintDetail
		statusarray []string
	)

	e := G_DbEngine
	statusarray = make([]string, 2)
	statusarray[0] = "N" // 新建
	statusarray[1] = "R" // 重打

	// 获取指定类型的开口打印头
	if err = e.Table("PrintHead").Where("PlantNr = ? and PrintFileType = ?",
		PlantNr, "EXCEL").In("Status", statusarray).Asc("CreateTime").Find(&datalst); err != nil {
		return
	}
	for i = 0; i < len(datalst); i++ {
		// 获取明细列表
		itemlst = []PrintDetail{}
		if err = e.Table("PrintDetail").Where("PlantNr = ? and PrintHeadId = ?",
			PlantNr, t.PrintHeadId).Find(&itemlst); err != nil {
			return
		}
		datalst[i].Itemlst = itemlst
	}

	return datalst, nil
}

//删除七天前的打印记录及其明细
func (t *PrintHead) DelBefore7DayRecords(daytime string) bool {
	e := G_DbEngine
	_, err := e.Table("PrintHead").Where("PlantNr = ? and CreateTime < ?", PlantNr, daytime).Delete(&PrintHead{})
	if err != nil {
		return false
	}
	_, err = e.Table("PrintDetail").Where("PlantNr = ? and CreateTime < ?", PlantNr, daytime).Delete(&PrintDetail{})
	if err != nil {
		return false
	}
	return true
}
